#ifndef COMMON_H
#define COMMON_H

#include <QtCore>

#define KB 1000.0
#define MB 1000000.0
#define GB 1000000000.0
#define TB 1000000000000.0
#define KiB 1024.0
#define MiB 1048576.0
#define GiB 1073741824.0
#define TiB 1099511627776.0

#define HOUR 3600

namespace BitBudget {

void daemonise();

void setup(QString dir, int n);

/*!
    \brief Returns the user's home directory.
*/
QByteArray home ();
/*!
    \brief Returns the user's XDG_DATA_HOME directory.
*/
QByteArray xdg_data_home ();
/*!
    \brief Returns the user's XDG_CONFIG_HOME directory.
*/
QByteArray xdg_config_home ();
/*!
    \brief Returns the BitBudget application data directory.
*/
QByteArray data_home ();
/*!
    \brief Returns the BitBudget configuration directory.
*/
QByteArray config_home ();
/*!
    \brief Returns a string representing a number of bytes in a human readable format using the decimal system.
*/
QByteArray format_bytes_decimal (ulong bytes);
/*!
    \brief Returns a string representing a number of bytes in a human readable format using the binary system.
*/
QByteArray format_bytes_binary (ulong bytes);
/*!
    \brief Round to 2dp.
*/
double round (double value);
/*!
    \brief Returns a string representing a number, rounded to 2dp.
*/
QByteArray format_number(double value);

}

#endif // COMMON_H
