#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QtCore>
#include <QtGui>
#include "config.h"
#include "database.h"
#include <QtDBus>
#include <dbusmenuexporter.h>

#include <QMainWindow>
#include <QGraphicsScene>
#include <QComboBox>


namespace Ui {
class MainWindow;
}

class MainWindow : public QMainWindow
{
    Q_OBJECT
    
public:
    explicit MainWindow(QWidget *parent = 0);
    ~MainWindow();
    QApplication *app;
    void refresh();
    void draw_chart();
    QGraphicsScene *scene;
    QTimer *timer;
    BitBudget::Config::ConfigFile config;
    ulong get_unit(QComboBox *w);
    int get_unit_index (ulong value);
    bool has_initialised;
    void process_arguments(QStringList args);
    bool _d;
    void go_preferences();
    void go_hourly(bool do_set);
    void go_daily(bool do_set);
    void go_monthly(bool do_set);
    void update_launcher(BitBudget::DataBase::overview_data o);
    QMenu *quicklist;
    DBusMenuExporter *quicklist_exporter;
    QVariantMap signal_properties;
    void unminimise();

    
private slots:
    void on_dateTimeEdit_dateTimeChanged(const QDateTime &date);

    void on_dateTimeEdit_2_dateTimeChanged(const QDateTime &date);

    void timerTimeout();

    void resizeEvent(QResizeEvent *event);

    void closeEvent(QCloseEvent *event);

    void on_comboBox_currentIndexChanged(int index);

    void on_dateTimeEdit_3_editingFinished();

    void on_dateTimeEdit_4_editingFinished();

    void onsettingschanged();

    void on_downquota_editingFinished();

    void on_upquota_editingFinished();

    void on_totalquota_editingFinished();

    void on_downquotaunit_currentIndexChanged(int index);

    void on_upquotaunit_currentIndexChanged(int index);

    void on_totalquotaunit_currentIndexChanged(int index);

    void on_monthrotate_editingFinished();

    void on_base2radio_toggled(bool checked);

    void on_base10radio_toggled(bool checked);

    void on_tabWidget_currentChanged(int index);


    void on_actionOverview_triggered();

    void on_actionCharts_triggered();

    void on_actionPreferences_triggered();

    void on_action_About_triggered();

    void on_actionReport_Bug_triggered();

    void on_actionOnline_Support_triggered();

    void quicklist_hourly_activate();
    void quicklist_daily_activate();
    void quicklist_monthly_activate();


private:
    Ui::MainWindow *ui;
};


class MainWindowAdaptor : public QDBusAbstractAdaptor
{
    Q_OBJECT
    Q_CLASSINFO("D-Bus Interface", "com.lazyteq.bitbudget.app.MainWindow")
public:
    MainWindowAdaptor(MainWindow *parent);
public slots:
    void process_argument(QString arg);
};

#endif // MAINWINDOW_H
