#include "common.h"
#include <sys/stat.h>
#include <unistd.h>


namespace BitBudget {

void daemonise () {
    pid_t pid, sid;
    pid = fork();
    if (pid < 0)
        exit(EXIT_FAILURE);
    if (pid > 0)
        exit(EXIT_SUCCESS);
    umask(0);
    sid = setsid();
    if (sid < 0)
        exit(EXIT_FAILURE);
    if ((chdir("/")) < 0)
        exit(EXIT_FAILURE);
    close(STDIN_FILENO);
    close(STDOUT_FILENO);
    close(STDERR_FILENO);
}

void setup (QString dir, int n) {
    if (n == 0 || n == 2)
        system((dir + "/bitbudget-daemon").toUtf8());
    QString startup = BitBudget::xdg_config_home() + "/autostart";
    QDir d;
    d.mkpath(startup);
    QFile f(startup.append("/bitbudget.desktop"));
    if (!f.exists()) {
        f.open(QIODevice::WriteOnly);
        QByteArray b = "[Desktop Entry]\nType=Application\nName=BitBudget\nExec=";
        b += dir.toUtf8() + "/BitBudget\n";
	b += "Icon=bitbudget\nX-GNOME-Autostart-enabled=true\nHidden=false\n";
        f.setPermissions(QFile::ReadOwner|QFile::WriteOwner|QFile::ExeOwner);
        f.write(b);
        f.flush();
        f.close();
    }
}

QByteArray home () {
    return qgetenv("HOME");
}

QByteArray xdg_data_home () {
    QByteArray path = qgetenv("XDG_DATA_HOME");
    if (path.isEmpty()) {
        path = home().append("/.local/share");
    }
    return path;
}

QByteArray xdg_config_home () {
    QByteArray path = qgetenv("XDG_DATA_HOME");
    if (path.isEmpty()) {
        path = home().append("/.config");
    }
    return path;
}

QByteArray data_home () {
    return xdg_data_home().append("/BitBudget");
}

QByteArray config_home () {
    return xdg_config_home().append("/bitbudget");
}

double round (double value) {
    return static_cast<double>(static_cast<int>(value*100+0.5))/100.0;
}

QByteArray format_number (double value) {
    QByteArray str;
    QTextStream out(&str);
    out << round(value);
    return str;
}

QByteArray format_bytes_decimal (ulong bytes) {
    QByteArray str;
    QByteArray unit;
    QTextStream out(&str);
    double number = bytes / TB;
    if (number >= 1) {
        unit = " TB";
    } else {
        number = bytes / GB;
        if (number >= 1) {
            unit = " GB";
        } else {
            number = bytes / MB;
            if (number >= 1) {
                unit = " MB";
            } else {
                number = bytes / KB;
                if (number >= 1) {
                    unit = " KB";
                } else {
                    number = bytes;
                    unit = " B";
                }
            }
        }
    }
    out << format_number(number) << unit;
    return str;
}

QByteArray format_bytes_binary (ulong bytes) {
    QByteArray str;
    QByteArray unit;
    QTextStream out(&str);
    double number = bytes / TiB;
    if (number >= 1) {
        unit = " TiB";
    } else {
        number = bytes / GiB;
        if (number >= 1) {
            unit = " GiB";
        } else {
            number = bytes / MiB;
            if (number >= 1) {
                unit = " MiB";
            } else {
                number = bytes / KiB;
                if (number >= 1) {
                    unit = " KiB";
                } else {
                    number = bytes;
                    unit = " B";
                }
            }
        }
    }
    out << format_number(number) << unit;
    return str;
}



}

