#include "common.h"
#include "config.h"
#include "networkmonitor.h"


BitBudget::Config::ConfigFile::ConfigFile() {
    binary = true;
    month_rotate = 1;
    download_quota = 0;
    upload_quota = 0;
    total_quota = 0;
    ifaces = BitBudget::NetworkMonitor::ifaces();
    indicator = true;
    notifications = true;
    QDir dir;
    dir.mkpath(BitBudget::xdg_config_home());
}

void BitBudget::Config::ConfigFile::load() {
    QFile f(BitBudget::config_home());
    QByteArray line;
    f.open(QFile::ReadOnly);
    if (f.isReadable()) {
        if (!f.atEnd()) {
            binary = f.readLine().startsWith("true");
        }
        if (!f.atEnd()) {
            line = f.readLine();
            line.chop(1);
            month_rotate = line.toInt();
            if (month_rotate < 1) {
                month_rotate = 1;
            }
            if (month_rotate > 28) {
                month_rotate = 28;
            }
        }
        if (!f.atEnd()) {
            line = f.readLine();
            line.chop(1);
            download_quota = line.toLong();
        }
        if (!f.atEnd()) {
            line = f.readLine();
            line.chop(1);
            upload_quota = line.toLong();
        }
        if (!f.atEnd()) {
            line = f.readLine();
            line.chop(1);
            total_quota = line.toLong();
        }
        if (!f.atEnd()) {
            line = f.readLine();
            line.chop(1);
            ifaces = line.split('\t');
        }
        if (!f.atEnd()) {
            indicator = f.readLine().startsWith("true");
        }
        if (!f.atEnd()) {
            notifications = f.readLine().startsWith("true");
        }
    }
}

void BitBudget::Config::ConfigFile::save() {
    QFile f(BitBudget::config_home());
    f.open(QFile::WriteOnly);
    QTextStream out(&f);
    if (binary) {
        out << "true\n";
    } else {
        out << "false\n";
    }
    out << month_rotate << "\n" << download_quota << "\n" << upload_quota << "\n" << total_quota << "\n";
    QListIterator<QByteArray> i(ifaces);
    while (i.hasNext()) {
        out << i.next();
        if (i.hasNext()) {
            out << "\t";
        }
    }
    out << "\n";
    if (indicator) {
        out << "true\n";
    } else {
        out << "false\n";
    }
    if (notifications) {
        out << "true\n";
    } else {
        out << "false\n";
    }
    out.flush();
    f.flush();
    f.close();
}

QByteArray BitBudget::Config::ConfigFile::format_bytes(ulong bytes) {
    if (this->binary) {
        return BitBudget::format_bytes_binary(bytes);
    }
    return BitBudget::format_bytes_decimal(bytes);
}



