#include "database.h"
#include "common.h"
#include "networkmonitor.h"


typedef BitBudget::DataBase::overview_data __db_ovd;

QList<QString> BitBudget::DataBase::list() {
    QDir dir(BitBudget::data_home());
    return dir.entryList();
}

QList<QDateTime> BitBudget::DataBase::list_dates() {
    QList<QString> l = BitBudget::DataBase::list();
    QListIterator<QString> i(l);
    QList<QDateTime> dts;
    while (i.hasNext()) {
        dts.append(QDateTime::fromString(i.next(), "yyyy-M-d_h"));
    }
    return dts;
}

BitBudget::DataBase::date_range BitBudget::DataBase::range () {
    BitBudget::DataBase::date_range dr;
    QList<QString> l = BitBudget::DataBase::list();
    QListIterator<QString> i(l);
    QString name;
    QDateTime dt;
    dr.start = QDateTime::currentDateTime();
    dr.finish = QDateTime::fromMSecsSinceEpoch(0);
    while (i.hasNext()) {
        name = i.next();
        dt = dt.fromString(name, "yyyy-M-d_h");
        if (dt < dr.start) {
            dr.start = dt;
        }
        if (dt > dr.finish) {
            dr.finish = dt;
        }
    }
    if (dr.start > dr.finish) {
        dr.finish = dr.start;
    }
    return dr;
}

QList<QDateTime> BitBudget::DataBase::hours (BitBudget::DataBase::date_range d) {
    QList<QDateTime> l;
    QDateTime current(d.start);
    while (current <= d.finish) {
        l.append(current);
        current = current.addSecs(HOUR);
    }
    return l;
}

QList<QDateTime> BitBudget::DataBase::days (BitBudget::DataBase::date_range d) {
    QList<QDateTime> l;
    QDateTime current(d.start);
    while (current <= d.finish) {
        l.append(current);
        current = current.addDays(1);
    }
    return l;
}


QList<QDateTime> BitBudget::DataBase::months (BitBudget::DataBase::date_range d, int rotate) {
    QList<QDateTime> l;
    BitBudget::DataBase::date_range current = BitBudget::DataBase::month(
                d.start.date().year(), d.start.date().month(), rotate);
    while (current.start <= d.finish) {
        l.append(current.start);
        current.finish = current.finish.addSecs(HOUR);
        current = BitBudget::DataBase::month(
                        current.finish.date().year(), current.finish.date().month(), rotate);
    }
    return l;
}

QList<QDateTime> BitBudget::DataBase::months (BitBudget::DataBase::date_range d) {
    return BitBudget::DataBase::months(d, 1);
}

BitBudget::DataBase::date_range BitBudget::DataBase::month (int year, int month, int n) {
    BitBudget::DataBase::date_range dr;
    if (n < 1) {
        n = 1;
    }
    if (n > 28) {
        n = 28;
    }
    int finish_month, finish_year;
    if (month == 12) {
        //handle year rollover
        finish_month = 1;
        finish_year = year + 1;
    } else {
        finish_month = month + 1;
        finish_year = year;
    }
    QDate start_date(year, month, n);
    QDate finish_date(finish_year, finish_month, n == 1 ? 1 : n - 1);
    if (n == 1) {
        //handle month starting on the 1st
        finish_date = finish_date.addDays(-1);
    }
    QTime start_time(0, 0);
    QTime finish_time(23, 0);
    dr.start.setDate(start_date);
    dr.start.setTime(start_time);
    dr.finish.setDate(finish_date);
    dr.finish.setTime(finish_time);
    return dr;
}

BitBudget::DataBase::date_range BitBudget::DataBase::month (int year, int month) {
    return BitBudget::DataBase::month(year, month, 1);
}

BitBudget::DataBase::date_range BitBudget::DataBase::day (int year, int month, int day) {
    BitBudget::DataBase::date_range dr;
    QDate start_date(year, month, day);
    QDate finish_date(year, month, day);
    QTime start_time(0, 0);
    QTime finish_time(23, 0);
    dr.start.setDate(start_date);
    dr.start.setTime(start_time);
    dr.finish.setDate(finish_date);
    dr.finish.setTime(finish_time);
    return dr;
}


QByteArray BitBudget::DataBase::path(QDateTime t) {
    QDate date = t.date();
    QTime time = t.time();
    return BitBudget::DataBase::path(date.year(), date.month(), date.day(), time.hour());
}

QByteArray BitBudget::DataBase::path(int year, int month, int day, int hour) {
    QByteArray p;
    QTextStream out(&p);
    out << BitBudget::data_home() << "/" << year << "-" << month << "-" << day << "_" << hour;
    return p;
}

__nwm_d BitBudget::DataBase::read(QDateTime t) {
    QDate date = t.date();
    QTime time = t.time();
    return BitBudget::DataBase::read(date.year(), date.month(), date.day(), time.hour());
}

__nwm_d BitBudget::DataBase::read(int year, int month, int day, int hour) {
    return BitBudget::NetworkMonitor::load_data(BitBudget::DataBase::path(year, month, day, hour));
}

void BitBudget::DataBase::write(__nwm_d d, QDateTime t) {
    QDate date = t.date();
    QTime time = t.time();
    return BitBudget::DataBase::write(d, date.year(), date.month(), date.day(), time.hour());
}

void BitBudget::DataBase::write(__nwm_d d, int year, int month, int day, int hour) {
    QByteArray p = BitBudget::DataBase::path(year, month, day, hour);
    QDir dir;
    dir.mkpath(BitBudget::data_home());
    BitBudget::NetworkMonitor::save_data(p, d);
}


__nwm_d BitBudget::DataBase::get_usage(BitBudget::DataBase::date_range range) {
//    QList<QDateTime> h = BitBudget::DataBase::hours(range);
//    QListIterator<QDateTime> i(h);
//    QDateTime d;
//    __nwm_d usage = BitBudget::NetworkMonitor::data_zero();
//    while (i.hasNext()) {
//        d = i.next();
//        usage = BitBudget::NetworkMonitor::data_sum(usage, BitBudget::DataBase::read(d));
//    }
//    return usage;
      QList<QDateTime> h = BitBudget::DataBase::list_dates();
      QListIterator<QDateTime> i(h);
      QDateTime d;
      __nwm_d usage = BitBudget::NetworkMonitor::data_zero();
      while (i.hasNext()) {
          d = i.next();
          if (d >= range.start && d <= range.finish) {
            usage = BitBudget::NetworkMonitor::data_sum(usage, BitBudget::DataBase::read(d));
          }
      }
      return usage;
}

__nwm_d BitBudget::DataBase::get_monthly_usage(int rotate, int year, int month) {
    return BitBudget::DataBase::get_usage(BitBudget::DataBase::month(year, month, rotate));
}
__nwm_d BitBudget::DataBase::get_monthly_usage(int rotate, int month) {
    QDate now = QDate::currentDate();
    if (month > 0) {
        now = now.addMonths(-month);
    }
    return BitBudget::DataBase::get_monthly_usage(rotate, now.year(), now.month());
}
__nwm_d BitBudget::DataBase::get_monthly_usage(int rotate) {
    return BitBudget::DataBase::get_monthly_usage(rotate, 0);
}
__nwm_d BitBudget::DataBase::get_monthly_usage() {
    return BitBudget::DataBase::get_monthly_usage(1);
}

__nwm_d BitBudget::DataBase::get_daily_usage(int year, int month, int day) {
    return BitBudget::DataBase::get_usage(BitBudget::DataBase::day(year, month, day));
}
__nwm_d BitBudget::DataBase::get_daily_usage(int day) {
    QDate now = QDate::currentDate();
    if (day > 0) {
        now = now.addDays(-day);
    }
    return BitBudget::DataBase::get_daily_usage(now.year(), now.month(), now.day());
}
__nwm_d BitBudget::DataBase::get_daily_usage() {
    return BitBudget::DataBase::get_daily_usage(0);
}

__nwm_d BitBudget::DataBase::get_hourly_usage(int year, int month, int day, int hour) {
    return BitBudget::DataBase::read(year, month, day, hour);
}
__nwm_d BitBudget::DataBase::get_hourly_usage(int hour) {
    QDateTime now = QDateTime::currentDateTime();
    if (hour > 0) {
        now = now.addSecs(-HOUR*hour);
    }
    return BitBudget::DataBase::read(now);
}
__nwm_d BitBudget::DataBase::get_hourly_usage() {
    return BitBudget::DataBase::read(QDateTime::currentDateTime());
}

__db_ovd BitBudget::DataBase::get_overview(int month_rotate, QList<QByteArray> ifaces) {
    __db_ovd ovd;
    BitBudget::NetworkMonitor::data_t d;
    d = BitBudget::NetworkMonitor::data_aggregate(
                BitBudget::DataBase::get_monthly_usage(month_rotate), ifaces);
    ovd.monthly_rx = d.rx;
    ovd.monthly_tx = d.tx;
    d = BitBudget::NetworkMonitor::data_aggregate(
                BitBudget::DataBase::get_daily_usage(), ifaces);
    ovd.daily_rx = d.rx;
    ovd.daily_tx = d.tx;
    d = BitBudget::NetworkMonitor::data_aggregate(
                BitBudget::DataBase::get_hourly_usage(), ifaces);
    ovd.hourly_rx = d.rx;
    ovd.hourly_tx = d.tx;
    return ovd;
}
__db_ovd BitBudget::DataBase::get_overview(QList<QByteArray> ifaces) {
    return BitBudget::DataBase::get_overview(0, ifaces);
}
__db_ovd BitBudget::DataBase::get_overview() {
    return BitBudget::DataBase::get_overview(0, BitBudget::NetworkMonitor::ifaces());
}



