#include "networkmonitor.h"


namespace BitBudget {
namespace NetworkMonitor {


data get_data (QObject &file) {
    data d;
    QIODevice *f = qobject_cast<QIODevice *>(&file);
    QList<QByteArray> line;
    data_t us;
    QByteArray iface;

    if (!f->isReadable()) {
        return d;
    }

    f->readLine();
    f->readLine();

    while (!f->atEnd()) {
        line = f->readLine().simplified().split(' ');
        iface = line.at(0);
        iface.chop(1);
        us.rx = line.at(1).toULong();
        us.tx = line.at(9).toULong();
        d.insert(iface, us);
    }

    f->close();

    return d;
}

data get_data (QString filename) {
    QFile f(filename);
    f.open(QFile::ReadOnly);
    return get_data(f);
}

data get_data () {
    return get_data("/proc/net/dev");
}

data data_delta (data a, data b) {
    data delta;
    QHashIterator<QByteArray, data_t> i(b);
    data_t us_a, us_b, us_delta;
    while (i.hasNext()) {
        i.next();
        us_b = i.value();
        us_a = a[i.key()];
        us_delta.rx = us_b.rx - us_a.rx;
        us_delta.tx = us_b.tx - us_a.tx;
        delta[i.key()] = us_delta;
    }
    return delta;
}

data data_sum (data a, data b) {
    data sum;
    QHashIterator<QByteArray, data_t> i(b);
    data_t us_a, us_b, us_sum;
    while (i.hasNext()) {
        i.next();
        us_b = i.value();
        us_a = a[i.key()];
        us_sum.rx = us_b.rx + us_a.rx;
        us_sum.tx = us_b.tx + us_a.tx;
        sum[i.key()] = us_sum;
    }
    return sum;
}

data data_delta (data initial) {
    return data_delta(initial, get_data());
}

data_t data_aggregate (data d, QList<QByteArray> ifaces) {
    data_t us;
    QListIterator<QByteArray> i(ifaces);
    QByteArray iface;
    us.rx = 0;
    us.tx = 0;
    while (i.hasNext()) {
        iface = i.next();
        us.rx += d[iface].rx;
        us.tx += d[iface].tx;
    }
    return us;
}

data_t data_aggregate (QList<QByteArray> ifaces) {
    return data_aggregate(get_data(), ifaces);
}

data data_zero () {
    data d;
    data_t us;
    QListIterator<QByteArray> i(ifaces_all());
    QByteArray iface;
    us.rx = 0;
    us.tx = 0;
    while (i.hasNext()) {
        iface = i.next();
        d[iface] = us;
    }
    return d;
}

data load_data (QObject &file) {
    data d = data_zero();
    QIODevice *f = qobject_cast<QIODevice *>(&file);
    QList<QByteArray> line;
    QByteArray iface;
    data_t us;

    if (f->isReadable()) {

        f->readLine();

        while (!f->atEnd()) {
            line = f->readLine().trimmed().split('\t');
            if (line.length() >= 3) {
                iface = line.at(0);
                us.rx = line.at(1).toULong();
                us.tx = line.at(2).toULong();
                d[iface] = us;
            }
        }

        f->close();

    }

    return d;
}

data load_data (QByteArray filename) {
    QFile f(filename);
    f.open(QFile::ReadOnly);
    return load_data(f);
}

void save_data (QObject &file, data &d) {
    QIODevice *f = qobject_cast<QIODevice *>(&file);
    QHashIterator<QByteArray, data_t> i(d);
    QString *line;

    if (!f->isWritable()) {
        return;
    }

    f->write("iface\trx\ttx\n");

    while (i.hasNext()) {
        i.next();
        line = new QString();
        QTextStream out(line, QIODevice::WriteOnly);
        out << i.key() << "\t" << i.value().rx << "\t" << i.value().tx << "\n";
        f->write(line->toUtf8());
        delete line;
    }

    f->close();
}

void save_data (QByteArray filename, data &d) {
    QFile f(filename);
    f.open(QFile::WriteOnly);
    save_data(f, d);
}

QList<QByteArray> ifaces_all () {
    return get_data().keys();
}

QList<QByteArray> ifaces () {
    QList<QByteArray> ia = ifaces_all();
    ia.removeAll("lo");
    return ia;
}

QList<QByteArray> ifaces (QList<QByteArray> exclude) {
    QList<QByteArray> ia = ifaces_all();
    QListIterator<QByteArray> i(exclude);
    while (i.hasNext()) {
        ia.removeAll(i.next());
    }
    return ia;
}



}
}
